@@ -1,5 +1,6 @@
 from datetime import timedelta
 from decimal import Decimal
+import re
 
 from dateutil.tz import tzlocal
 import numpy as np
@@ -8,6 +9,8 @@ import pytest
 from pandas.compat import is_platform_windows
 import pandas.util._test_decorators as td
 
+from pandas.core.dtypes.common import is_categorical_dtype
+
 import pandas as pd
 from pandas import (
     Categorical,
@@ -43,7 +46,7 @@ def assert_stat_op_calc(
 
     Parameters
     ----------
-    opname : string
+    opname : str
         Name of the operator to test on frame
     alternative : function
         Function that opname is tested against; i.e. "frame.opname()" should
@@ -90,7 +93,7 @@ def assert_stat_op_calc(
         tm.assert_series_equal(
             result0, frame.apply(wrapper), check_dtype=check_dtype, rtol=rtol, atol=atol
         )
-        # HACK: win32
+        # FIXME: HACK: win32
         tm.assert_series_equal(
             result1,
             frame.apply(wrapper, axis=1),
@@ -140,13 +143,13 @@ def assert_stat_op_calc(
             tm.assert_series_equal(r1, expected)
 
 
-def assert_stat_op_api(opname, float_frame, float_string_frame, has_numeric_only=False):
+def assert_stat_op_api(opname, float_frame, float_string_frame, has_numeric_only=True):
     """
     Check that API for operator opname works as advertised on frame
 
     Parameters
     ----------
-    opname : string
+    opname : str
         Name of the operator to test on frame
     float_frame : DataFrame
         DataFrame with columns of type float
@@ -172,7 +175,7 @@ def assert_bool_op_calc(opname, alternative, frame, has_skipna=True):
 
     Parameters
     ----------
-    opname : string
+    opname : str
         Name of the operator to test on frame
     alternative : function
         Function that opname is tested against; i.e. "frame.opname()" should
@@ -199,7 +202,7 @@ def assert_bool_op_calc(opname, alternative, frame, has_skipna=True):
         tm.assert_series_equal(result0, frame.apply(wrapper))
         tm.assert_series_equal(
             result1, frame.apply(wrapper, axis=1), check_dtype=False
-        )  # HACK: win32
+        )  # FIXME: HACK: win32
     else:
         skipna_wrapper = alternative
         wrapper = alternative
@@ -237,7 +240,7 @@ def assert_bool_op_api(
 
     Parameters
     ----------
-    opname : string
+    opname : str
         Name of the operator to test on frame
     float_frame : DataFrame
         DataFrame with columns of type float
@@ -249,6 +252,7 @@ def assert_bool_op_api(
     # make sure op works on mixed-type frame
     mixed = float_string_frame
     mixed["_bool_"] = np.random.randn(len(mixed)) > 0.5
+
     getattr(mixed, opname)(axis=0)
     getattr(mixed, opname)(axis=1)
 
@@ -264,28 +268,32 @@ class TestDataFrameAnalytics:
     # ---------------------------------------------------------------------
     # Reductions
 
+    @pytest.mark.filterwarnings("ignore:Dropping of nuisance:FutureWarning")
     def test_stat_op_api(self, float_frame, float_string_frame):
-        assert_stat_op_api(
-            "count", float_frame, float_string_frame, has_numeric_only=True
-        )
-        assert_stat_op_api(
-            "sum", float_frame, float_string_frame, has_numeric_only=True
-        )
+        assert_stat_op_api("count", float_frame, float_string_frame)
+        assert_stat_op_api("sum", float_frame, float_string_frame)
 
-        assert_stat_op_api("nunique", float_frame, float_string_frame)
+        assert_stat_op_api(
+            "nunique", float_frame, float_string_frame, has_numeric_only=False
+        )
         assert_stat_op_api("mean", float_frame, float_string_frame)
         assert_stat_op_api("product", float_frame, float_string_frame)
         assert_stat_op_api("median", float_frame, float_string_frame)
         assert_stat_op_api("min", float_frame, float_string_frame)
         assert_stat_op_api("max", float_frame, float_string_frame)
-        assert_stat_op_api("mad", float_frame, float_string_frame)
+        assert_stat_op_api(
+            "mad", float_frame, float_string_frame, has_numeric_only=False
+        )
         assert_stat_op_api("var", float_frame, float_string_frame)
         assert_stat_op_api("std", float_frame, float_string_frame)
         assert_stat_op_api("sem", float_frame, float_string_frame)
         assert_stat_op_api("median", float_frame, float_string_frame)
 
         try:
-            from scipy.stats import kurtosis, skew  # noqa:F401
+            from scipy.stats import (  # noqa:F401
+                kurtosis,
+                skew,
+            )
 
             assert_stat_op_api("skew", float_frame, float_string_frame)
             assert_stat_op_api("kurt", float_frame, float_string_frame)
@@ -368,7 +376,10 @@ class TestDataFrameAnalytics:
         )
 
         try:
-            from scipy import kurtosis, skew  # noqa:F401
+            from scipy import (  # noqa:F401
+                kurtosis,
+                skew,
+            )
 
             assert_stat_op_calc("skew", skewness, float_frame_with_na)
             assert_stat_op_calc("kurt", kurt, float_frame_with_na)
@@ -376,7 +387,8 @@ class TestDataFrameAnalytics:
             pass
 
     # TODO: Ensure warning isn't emitted in the first place
-    @pytest.mark.filterwarnings("ignore:All-NaN:RuntimeWarning")
+    # ignore mean of empty slice and all-NaN
+    @pytest.mark.filterwarnings("ignore::RuntimeWarning")
     def test_median(self, float_frame_with_na, int_frame):
         def wrapper(x):
             if isna(x).any():
@@ -428,12 +440,17 @@ class TestDataFrameAnalytics:
                 "str": ["a", "b", "c", "d"],
             }
         )
-
-        result = getattr(df, op)()
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            result = getattr(df, op)()
         assert len(result) == 2
 
         with pd.option_context("use_bottleneck", False):
-            result = getattr(df, op)()
+            with tm.assert_produces_warning(
+                FutureWarning, match="Select only valid columns"
+            ):
+                result = getattr(df, op)()
             assert len(result) == 2
 
     def test_reduce_mixed_frame(self):
@@ -450,7 +467,8 @@ class TestDataFrameAnalytics:
         tm.assert_numpy_array_equal(
             test.values, np.array([2, 150, "abcde"], dtype=object)
         )
-        tm.assert_series_equal(test, df.T.sum(axis=1))
+        alt = df.T.sum(axis=1)
+        tm.assert_series_equal(test, alt)
 
     def test_nunique(self):
         df = DataFrame({"A": [1, 1, 1], "B": [1, 2, 3], "C": [1, np.nan, 3]})
@@ -503,7 +521,10 @@ class TestDataFrameAnalytics:
 
         df = DataFrame(d)
 
-        result = df.mean()
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            result = df.mean()
         expected = Series([2.7, 681.6], index=["A", "C"])
         tm.assert_series_equal(result, expected)
 
@@ -573,7 +594,8 @@ class TestDataFrameAnalytics:
         df = DataFrame(np.random.randn(6, 3), index=index)
 
         kurt = df.kurt()
-        kurt2 = df.kurt(level=0).xs("bar")
+        with tm.assert_produces_warning(FutureWarning):
+            kurt2 = df.kurt(level=0).xs("bar")
         tm.assert_series_equal(kurt, kurt2, check_names=False)
         assert kurt.name is None
         assert kurt2.name == "bar"
@@ -663,12 +685,18 @@ class TestDataFrameAnalytics:
         df = DataFrame({"A": [np.nan, np.nan, "a", "a"]})
         expected = DataFrame({"A": ["a", np.nan]})
 
-        with tm.assert_produces_warning(UserWarning, check_stacklevel=False):
+        with tm.assert_produces_warning(UserWarning):
             result = df.mode(dropna=False)
             result = result.sort_values(by="A").reset_index(drop=True)
 
         tm.assert_frame_equal(result, expected)
 
+    def test_mode_empty_df(self):
+        df = DataFrame([], columns=["a", "b"])
+        result = df.mode()
+        expected = DataFrame([], columns=["a", "b"], index=Index([], dtype=int))
+        tm.assert_frame_equal(result, expected)
+
     def test_operators_timedelta64(self):
         df = DataFrame(
             {
@@ -726,7 +754,8 @@ class TestDataFrameAnalytics:
         tm.assert_series_equal(result, expected)
 
         # excludes numeric
-        result = mixed.min(axis=1)
+        with tm.assert_produces_warning(FutureWarning, match="Select only valid"):
+            result = mixed.min(axis=1)
         expected = Series([1, 1, 1.0], index=[0, 1, 2])
         tm.assert_series_equal(result, expected)
 
@@ -783,34 +812,36 @@ class TestDataFrameAnalytics:
         assert len(axis1) == 0
 
     @pytest.mark.parametrize("method, unit", [("sum", 0), ("prod", 1)])
-    def test_sum_prod_nanops(self, method, unit):
+    @pytest.mark.parametrize("numeric_only", [None, True, False])
+    def test_sum_prod_nanops(self, method, unit, numeric_only):
         idx = ["a", "b", "c"]
         df = DataFrame({"a": [unit, unit], "b": [unit, np.nan], "c": [np.nan, np.nan]})
         # The default
-        result = getattr(df, method)
+        result = getattr(df, method)(numeric_only=numeric_only)
         expected = Series([unit, unit, unit], index=idx, dtype="float64")
+        tm.assert_series_equal(result, expected)
 
         # min_count=1
-        result = getattr(df, method)(min_count=1)
+        result = getattr(df, method)(numeric_only=numeric_only, min_count=1)
         expected = Series([unit, unit, np.nan], index=idx)
         tm.assert_series_equal(result, expected)
 
         # min_count=0
-        result = getattr(df, method)(min_count=0)
+        result = getattr(df, method)(numeric_only=numeric_only, min_count=0)
         expected = Series([unit, unit, unit], index=idx, dtype="float64")
         tm.assert_series_equal(result, expected)
 
-        result = getattr(df.iloc[1:], method)(min_count=1)
+        result = getattr(df.iloc[1:], method)(numeric_only=numeric_only, min_count=1)
         expected = Series([unit, np.nan, np.nan], index=idx)
         tm.assert_series_equal(result, expected)
 
         # min_count > 1
         df = DataFrame({"A": [unit] * 10, "B": [unit] * 5 + [np.nan] * 5})
-        result = getattr(df, method)(min_count=5)
+        result = getattr(df, method)(numeric_only=numeric_only, min_count=5)
         expected = Series(result, index=["A", "B"])
         tm.assert_series_equal(result, expected)
 
-        result = getattr(df, method)(min_count=6)
+        result = getattr(df, method)(numeric_only=numeric_only, min_count=6)
         expected = Series(result, index=["A", "B"])
         tm.assert_series_equal(result, expected)
 
@@ -819,7 +850,7 @@ class TestDataFrameAnalytics:
         idx = ["a", "b", "c"]
         df = DataFrame({"a": [0, 0], "b": [0, np.nan], "c": [np.nan, np.nan]})
 
-        df2 = df.apply(pd.to_timedelta)
+        df2 = df.apply(to_timedelta)
 
         # 0 by default
         result = df2.sum()
@@ -835,6 +866,13 @@ class TestDataFrameAnalytics:
         expected = Series([0, 0, np.nan], dtype="m8[ns]", index=idx)
         tm.assert_series_equal(result, expected)
 
+    def test_sum_nanops_min_count(self):
+        # https://github.com/pandas-dev/pandas/issues/39738
+        df = DataFrame({"x": [1, 2, 3], "y": [4, 5, 6]})
+        result = df.sum(min_count=10)
+        expected = Series([np.nan, np.nan], index=["x", "y"])
+        tm.assert_series_equal(result, expected)
+
     def test_sum_object(self, float_frame):
         values = float_frame.values.astype(int)
         frame = DataFrame(values, index=float_frame.index, columns=float_frame.columns)
@@ -849,23 +887,26 @@ class TestDataFrameAnalytics:
 
     def test_sum_mixed_datetime(self):
         # GH#30886
-        df = DataFrame(
-            {"A": pd.date_range("2000", periods=4), "B": [1, 2, 3, 4]}
-        ).reindex([2, 3, 4])
-        result = df.sum()
+        df = DataFrame({"A": date_range("2000", periods=4), "B": [1, 2, 3, 4]}).reindex(
+            [2, 3, 4]
+        )
+        with tm.assert_produces_warning(FutureWarning, match="Select only valid"):
+            result = df.sum()
 
         expected = Series({"B": 7.0})
         tm.assert_series_equal(result, expected)
 
     def test_mean_corner(self, float_frame, float_string_frame):
         # unit test when have object data
-        the_mean = float_string_frame.mean(axis=0)
+        with tm.assert_produces_warning(FutureWarning, match="Select only valid"):
+            the_mean = float_string_frame.mean(axis=0)
         the_sum = float_string_frame.sum(axis=0, numeric_only=True)
         tm.assert_index_equal(the_sum.index, the_mean.index)
         assert len(the_mean.index) < len(float_string_frame.columns)
 
         # xs sum mixed type, just want to know it works...
-        the_mean = float_string_frame.mean(axis=1)
+        with tm.assert_produces_warning(FutureWarning, match="Select only valid"):
+            the_mean = float_string_frame.mean(axis=1)
         the_sum = float_string_frame.sum(axis=1, numeric_only=True)
         tm.assert_index_equal(the_sum.index, the_mean.index)
 
@@ -881,7 +922,7 @@ class TestDataFrameAnalytics:
         df = DataFrame(
             {
                 "A": np.arange(3),
-                "B": pd.date_range("2016-01-01", periods=3),
+                "B": date_range("2016-01-01", periods=3),
                 "C": pd.timedelta_range("1D", periods=3),
                 "D": pd.period_range("2016", periods=3, freq="A"),
             }
@@ -900,7 +941,7 @@ class TestDataFrameAnalytics:
         df = DataFrame(
             {
                 "A": np.arange(3),
-                "B": pd.date_range("2016-01-01", periods=3),
+                "B": date_range("2016-01-01", periods=3),
                 "C": pd.timedelta_range("1D", periods=3),
             }
         )
@@ -926,10 +967,13 @@ class TestDataFrameAnalytics:
 
     def test_stats_mixed_type(self, float_string_frame):
         # don't blow up
-        float_string_frame.std(1)
-        float_string_frame.var(1)
-        float_string_frame.mean(1)
-        float_string_frame.skew(1)
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            float_string_frame.std(1)
+            float_string_frame.var(1)
+            float_string_frame.mean(1)
+            float_string_frame.skew(1)
 
     def test_sum_bools(self):
         df = DataFrame(index=range(1), columns=range(10))
@@ -971,7 +1015,7 @@ class TestDataFrameAnalytics:
 
     def test_idxmax_mixed_dtype(self):
         # don't cast to object, which would raise in nanops
-        dti = pd.date_range("2016-01-01", periods=3)
+        dti = date_range("2016-01-01", periods=3)
 
         df = DataFrame({1: [0, 2, 1], 2: range(3)[::-1], 3: dti})
 
@@ -983,18 +1027,81 @@ class TestDataFrameAnalytics:
         expected = Series([0, 2, 0], index=[1, 2, 3])
         tm.assert_series_equal(result, expected)
 
+        # with NaTs
+        df.loc[0, 3] = pd.NaT
+        result = df.idxmax()
+        expected = Series([1, 0, 2], index=[1, 2, 3])
+        tm.assert_series_equal(result, expected)
+
+        result = df.idxmin()
+        expected = Series([0, 2, 1], index=[1, 2, 3])
+        tm.assert_series_equal(result, expected)
+
+        # with multi-column dt64 block
+        df[4] = dti[::-1]
+        df._consolidate_inplace()
+
+        result = df.idxmax()
+        expected = Series([1, 0, 2, 0], index=[1, 2, 3, 4])
+        tm.assert_series_equal(result, expected)
+
+        result = df.idxmin()
+        expected = Series([0, 2, 1, 2], index=[1, 2, 3, 4])
+        tm.assert_series_equal(result, expected)
+
+    @pytest.mark.parametrize(
+        "op, expected_value",
+        [("idxmax", [0, 4]), ("idxmin", [0, 5])],
+    )
+    def test_idxmax_idxmin_convert_dtypes(self, op, expected_value):
+        # GH 40346
+        df = DataFrame(
+            {
+                "ID": [100, 100, 100, 200, 200, 200],
+                "value": [0, 0, 0, 1, 2, 0],
+            },
+            dtype="Int64",
+        )
+        df = df.groupby("ID")
+
+        result = getattr(df, op)()
+        expected = DataFrame(
+            {"value": expected_value},
+            index=Index([100, 200], dtype="object", name="ID"),
+        )
+        tm.assert_frame_equal(result, expected)
+
+    def test_idxmax_dt64_multicolumn_axis1(self):
+        dti = date_range("2016-01-01", periods=3)
+        df = DataFrame({3: dti, 4: dti[::-1]})
+        df.iloc[0, 0] = pd.NaT
+
+        df._consolidate_inplace()
+
+        result = df.idxmax(axis=1)
+        expected = Series([4, 3, 3])
+        tm.assert_series_equal(result, expected)
+
+        result = df.idxmin(axis=1)
+        expected = Series([4, 3, 4])
+        tm.assert_series_equal(result, expected)
+
     # ----------------------------------------------------------------------
     # Logical reductions
 
     @pytest.mark.parametrize("opname", ["any", "all"])
     def test_any_all(self, opname, bool_frame_with_na, float_string_frame):
-        assert_bool_op_calc(
-            opname, getattr(np, opname), bool_frame_with_na, has_skipna=True
-        )
         assert_bool_op_api(
             opname, bool_frame_with_na, float_string_frame, has_bool_only=True
         )
 
+    @pytest.mark.parametrize("opname", ["any", "all"])
+    def test_any_all_bool_frame(self, opname, bool_frame_with_na):
+        # GH#12863: numpy gives back non-boolean data for object type
+        # so fill NaNs to compare with pandas behavior
+        df = bool_frame_with_na.fillna(True)
+        assert_bool_op_calc(opname, getattr(np, opname), df, has_skipna=True)
+
     def test_any_all_extra(self):
         df = DataFrame(
             {
@@ -1028,6 +1135,23 @@ class TestDataFrameAnalytics:
         result = df[["C"]].all(axis=None).item()
         assert result is True
 
+    @pytest.mark.parametrize("axis", [0, 1])
+    @pytest.mark.parametrize("bool_agg_func", ["any", "all"])
+    @pytest.mark.parametrize("skipna", [True, False])
+    def test_any_all_object_dtype(self, axis, bool_agg_func, skipna):
+        # GH#35450
+        df = DataFrame(
+            data=[
+                [1, np.nan, np.nan, True],
+                [np.nan, 2, np.nan, True],
+                [np.nan, np.nan, np.nan, True],
+                [np.nan, np.nan, "5", np.nan],
+            ]
+        )
+        result = getattr(df, bool_agg_func)(axis=axis, skipna=skipna)
+        expected = Series([True, True, True, True])
+        tm.assert_series_equal(result, expected)
+
     def test_any_datetime(self):
 
         # GH 23070
@@ -1122,12 +1246,23 @@ class TestDataFrameAnalytics:
     def test_any_all_np_func(self, func, data, expected):
         # GH 19976
         data = DataFrame(data)
-        result = func(data)
+
+        warn = None
+        if any(is_categorical_dtype(x) for x in data.dtypes):
+            warn = FutureWarning
+
+        with tm.assert_produces_warning(
+            warn, match="Select only valid columns", check_stacklevel=False
+        ):
+            result = func(data)
         assert isinstance(result, np.bool_)
         assert result.item() is expected
 
         # method version
-        result = getattr(DataFrame(data), func.__name__)(axis=None)
+        with tm.assert_produces_warning(
+            warn, match="Select only valid columns", check_stacklevel=False
+        ):
+            result = getattr(DataFrame(data), func.__name__)(axis=None)
         assert isinstance(result, np.bool_)
         assert result.item() is expected
 
@@ -1144,6 +1279,9 @@ class TestDataFrameAnalytics:
         df._consolidate_inplace()
         df["C"] = Series([True, True])
 
+        # Categorical of bools is _not_ considered booly
+        df["D"] = df["C"].astype("category")
+
         # The underlying bug is in DataFrame._get_bool_data, so we check
         #  that while we're here
         res = df._get_bool_data()
@@ -1180,7 +1318,8 @@ class TestDataFrameAnalytics:
         )
         xpr = "Must specify 'axis' when aggregating by level."
         with pytest.raises(ValueError, match=xpr):
-            getattr(df, method)(axis=None, level="out")
+            with tm.assert_produces_warning(FutureWarning):
+                getattr(df, method)(axis=None, level="out")
 
     # ---------------------------------------------------------------------
     # Unsorted
@@ -1223,13 +1362,15 @@ class TestDataFrameReductions:
         exp = Series([pd.NaT], index=["foo"])
         tm.assert_series_equal(res, exp)
 
-    def test_min_max_dt64_with_NaT_skipna_false(self, tz_naive_fixture):
+    def test_min_max_dt64_with_NaT_skipna_false(self, request, tz_naive_fixture):
         # GH#36907
         tz = tz_naive_fixture
         if isinstance(tz, tzlocal) and is_platform_windows():
-            pytest.xfail(
-                reason="GH#37659 OSError raised within tzlocal bc Windows "
-                "chokes in times before 1970-01-01"
+            request.node.add_marker(
+                pytest.mark.xfail(
+                    reason="GH#37659 OSError raised within tzlocal bc Windows "
+                    "chokes in times before 1970-01-01"
+                )
             )
 
         df = DataFrame(
@@ -1241,7 +1382,6 @@ class TestDataFrameReductions:
                 "b": [Timestamp("2020-02-01 08:00:00", tz=tz), pd.NaT],
             }
         )
-
         res = df.min(axis=1, skipna=False)
         expected = Series([df.loc[0, "a"], pd.NaT])
         assert expected.dtype == df["a"].dtype
@@ -1259,8 +1399,8 @@ class TestDataFrameReductions:
         # returned NaT for series. These tests check that the API is consistent in
         # min/max calls on empty Series/DataFrames. See GH:33704 for more
         # information
-        df = DataFrame({"x": pd.to_datetime([])})
-        expected_dt_series = Series(pd.to_datetime([]))
+        df = DataFrame({"x": to_datetime([])})
+        expected_dt_series = Series(to_datetime([]))
         # check axis 0
         assert (df.min(axis=0).x is pd.NaT) == (expected_dt_series.min() is pd.NaT)
         assert (df.max(axis=0).x is pd.NaT) == (expected_dt_series.max() is pd.NaT)
@@ -1288,7 +1428,7 @@ class TestDataFrameReductions:
     @pytest.mark.parametrize("method", ["min", "max"])
     def test_preserve_timezone(self, initial: str, method):
         # GH 28552
-        initial_dt = pd.to_datetime(initial)
+        initial_dt = to_datetime(initial)
         expected = Series([initial_dt])
         df = DataFrame([expected])
         result = getattr(df, method)(axis=1)
@@ -1303,11 +1443,13 @@ class TestDataFrameReductions:
             ],
         )
 
-        result = df.any(level=0)
+        with tm.assert_produces_warning(FutureWarning, match="Using the level"):
+            result = df.any(level=0)
         ex = DataFrame({"data": [False, True]}, index=["one", "two"])
         tm.assert_frame_equal(result, ex)
 
-        result = df.all(level=0)
+        with tm.assert_produces_warning(FutureWarning, match="Using the level"):
+            result = df.all(level=0)
         ex = DataFrame({"data": [False, False]}, index=["one", "two"])
         tm.assert_frame_equal(result, ex)
 
@@ -1316,7 +1458,7 @@ class TestDataFrameReductions:
         df = DataFrame(
             {
                 "a": Series([0, 0]),
-                "t": Series([pd.to_timedelta(0, "s"), pd.to_timedelta(1, "ms")]),
+                "t": Series([to_timedelta(0, "s"), to_timedelta(1, "ms")]),
             }
         )
 
@@ -1328,6 +1470,34 @@ class TestDataFrameReductions:
         expected = Series(data=[False, True])
         tm.assert_series_equal(result, expected)
 
+    @pytest.mark.parametrize(
+        "func",
+        [
+            "any",
+            "all",
+            "count",
+            "sum",
+            "prod",
+            "max",
+            "min",
+            "mean",
+            "median",
+            "skew",
+            "kurt",
+            "sem",
+            "var",
+            "std",
+            "mad",
+        ],
+    )
+    def test_reductions_deprecation_level_argument(self, frame_or_series, func):
+        # GH#39983
+        obj = frame_or_series(
+            [1, 2, 3], index=MultiIndex.from_arrays([[1, 2, 3], [4, 5, 6]])
+        )
+        with tm.assert_produces_warning(FutureWarning, match="level"):
+            getattr(obj, func)(level=0)
+
 
 class TestNuisanceColumns:
     @pytest.mark.parametrize("method", ["any", "all"])
@@ -1348,11 +1518,17 @@ class TestNuisanceColumns:
 
         # With bool_only=None, operating on this column raises and is ignored,
         #  so we expect an empty result.
-        result = getattr(df, method)(bool_only=None)
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            result = getattr(df, method)(bool_only=None)
         expected = Series([], index=Index([]), dtype=bool)
         tm.assert_series_equal(result, expected)
 
-        result = getattr(np, method)(df, axis=0)
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns", check_stacklevel=False
+        ):
+            result = getattr(np, method)(df, axis=0)
         tm.assert_series_equal(result, expected)
 
     def test_median_categorical_dtype_nuisance_column(self):
@@ -1367,7 +1543,10 @@ class TestNuisanceColumns:
         with pytest.raises(TypeError, match="does not implement reduction"):
             df.median(numeric_only=False)
 
-        result = df.median()
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            result = df.median()
         expected = Series([], index=Index([]), dtype=np.float64)
         tm.assert_series_equal(result, expected)
 
@@ -1377,7 +1556,10 @@ class TestNuisanceColumns:
         with pytest.raises(TypeError, match="does not implement reduction"):
             df.median(numeric_only=False)
 
-        result = df.median()
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            result = df.median()
         expected = Series([2.0], index=["B"])
         tm.assert_series_equal(result, expected)
 
@@ -1401,23 +1583,35 @@ class TestNuisanceColumns:
         with pytest.raises(TypeError, match="is not ordered for operation"):
             getattr(df, method)(numeric_only=False)
 
-        result = getattr(df, method)()
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            result = getattr(df, method)()
         expected = Series([], index=Index([]), dtype=np.float64)
         tm.assert_series_equal(result, expected)
 
-        result = getattr(np, method)(df)
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns", check_stacklevel=False
+        ):
+            result = getattr(np, method)(df)
         tm.assert_series_equal(result, expected)
 
         # same thing, but with an additional non-categorical column
         df["B"] = df["A"].astype(object)
-        result = getattr(df, method)()
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            result = getattr(df, method)()
         if method == "min":
             expected = Series(["a"], index=["B"])
         else:
             expected = Series(["c"], index=["B"])
         tm.assert_series_equal(result, expected)
 
-        result = getattr(np, method)(df)
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns", check_stacklevel=False
+        ):
+            result = getattr(np, method)(df)
         tm.assert_series_equal(result, expected)
 
     def test_reduction_object_block_splits_nuisance_columns(self):
@@ -1425,14 +1619,20 @@ class TestNuisanceColumns:
         df = DataFrame({"A": [0, 1, 2], "B": ["a", "b", "c"]}, dtype=object)
 
         # We should only exclude "B", not "A"
-        result = df.mean()
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            result = df.mean()
         expected = Series([1.0], index=["A"])
         tm.assert_series_equal(result, expected)
 
         # Same behavior but heterogeneous dtype
         df["C"] = df["A"].astype(int) + 4
 
-        result = df.mean()
+        with tm.assert_produces_warning(
+            FutureWarning, match="Select only valid columns"
+        ):
+            result = df.mean()
         expected = Series([1.0, 5.0], index=["A", "C"])
         tm.assert_series_equal(result, expected)
 
@@ -1484,3 +1684,42 @@ def test_minmax_extensionarray(method, numeric_only):
         [getattr(int64_info, method)], index=Index(["Int64"], dtype="object")
     )
     tm.assert_series_equal(result, expected)
+
+
+@pytest.mark.parametrize("meth", ["max", "min", "sum", "mean", "median"])
+def test_groupby_regular_arithmetic_equivalent(meth):
+    # GH#40660
+    df = DataFrame(
+        {"a": [pd.Timedelta(hours=6), pd.Timedelta(hours=7)], "b": [12.1, 13.3]}
+    )
+    expected = df.copy()
+
+    with tm.assert_produces_warning(FutureWarning):
+        result = getattr(df, meth)(level=0)
+    tm.assert_frame_equal(result, expected)
+
+    result = getattr(df.groupby(level=0), meth)(numeric_only=False)
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize("ts_value", [Timestamp("2000-01-01"), pd.NaT])
+def test_frame_mixed_numeric_object_with_timestamp(ts_value):
+    # GH 13912
+    df = DataFrame({"a": [1], "b": [1.1], "c": ["foo"], "d": [ts_value]})
+    with tm.assert_produces_warning(FutureWarning, match="Dropping of nuisance"):
+        result = df.sum()
+    expected = Series([1, 1.1, "foo"], index=list("abc"))
+    tm.assert_series_equal(result, expected)
+
+
+def test_prod_sum_min_count_mixed_object():
+    # https://github.com/pandas-dev/pandas/issues/41074
+    df = DataFrame([1, "a", True])
+
+    result = df.prod(axis=0, min_count=1, numeric_only=False)
+    expected = Series(["a"])
+    tm.assert_series_equal(result, expected)
+
+    msg = re.escape("unsupported operand type(s) for +: 'int' and 'str'")
+    with pytest.raises(TypeError, match=msg):
+        df.sum(axis=0, min_count=1, numeric_only=False)
